#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import dbus
import dbus.service
import logging
import sys
from gettext import gettext as _

from RevisionsManager.Core.RMVersionManager import RMVersionManager
from RevisionsManager.Core.RMOsInfo import RMOsInfo
from RevisionsManager.Core.RMRemote import RMRemote
from RevisionsManager.Core.RMClear import RMClear
from RevisionsManager.Core.RMDaq import RMDaq


RM_DBUS_INTERFACE = 'com.kylin.RevisionsManager.interface'
RM_DBUS_PATH = '/com/kylin/RevisionsManager'
RM_DBUS_SERVICE = 'com.kylin.RevisionsManager'

# method 加锁防止调用混乱
class RevisionsManagerDbus(dbus.service.Object):
    """ Provide the RevisionsManagerIFace """
    def __init__(self):
        self.methodLock = False             # Dbus method lock
        self.runningMethod = None           # Dbus method running name
        self.bus = dbus.SystemBus()
        try:
            bus_name = dbus.service.BusName(RM_DBUS_SERVICE,
                                            self.bus,
                                            do_not_queue=True)
            dbus.service.Object.__init__(self, bus_name, RM_DBUS_PATH)
            logging.info("initiate dbus success ...")
        except dbus.exceptions.NameExistsException:
            logging.critical("Another RevisionsManagerDaemon is already running")
            sys.exit(1)

        self.VersionManagerHD = RMVersionManager()
        self.VersionManagerHD.reg_up2_process_func(self.upgradeProcessSig2)
        self.VersionManagerHD.reg_up_process_func(self.upgradeProcessSig)
        self.VersionManagerHD.reg_upCheck_process_func(self.upgradeCheckSig)
        self.VersionManagerHD.reg_back_process_func(self.fallbackProcessSig)
        self.VersionManagerHD.reg_backCheck_process_func(self.fallbackCheckSig)
        self.osInfoHD = RMOsInfo()
        self.rmRemote = RMRemote()
        self.rmRemote.reg_check_process_func(self.checkProcessSig)
        self.rmRemote.reg_download_process_func(self.downloadProcessSig)

        self.rmClear = RMClear()
        self.daqInfo = RMDaq()
        logging.info("[DBUS]: init finished")


    # 升级第二阶段进度信号
    @dbus.service.signal(RM_DBUS_INTERFACE, signature="si")
    def upgradeProcessSig2(self, itermStr, process):
        logging.info("Emitting upgradeProcess2 item[%s] process[%d].\n",itermStr, process)
    
    @dbus.service.signal(RM_DBUS_INTERFACE, signature="siis")
    def upgradeCheckSig(self, itermStr, state, process, suggestion):
        logging.info("Emitting upgradeCheck item[%s] state[%d] process[%d] suggestion[%s].\n",itermStr, state, process, suggestion)

    @dbus.service.signal(RM_DBUS_INTERFACE, signature="siis")
    def upgradeProcessSig(self, itermStr, state, process, suggestion):
        logging.info("Emitting upgradeProcess item[%s] state[%d] process[%d] suggestion[%s].\n",itermStr, state, process, suggestion)

    @dbus.service.signal(RM_DBUS_INTERFACE, signature="siis")
    def fallbackCheckSig(self, itermStr, state, process, suggestion):
        logging.info("Emitting fallbackCheck item[%s] state[%d] process[%d] suggestion[%s].\n",itermStr, state, process, suggestion)

    @dbus.service.signal(RM_DBUS_INTERFACE, signature="siis")
    def fallbackProcessSig(self, itermStr, state, process, suggestion):
        logging.info("Emitting fallbackProcess item[%s] state[%d] process[%d] suggestion[%s].\n",itermStr, state, process, suggestion)

    #升级前检测
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature="s")
    def preUpgradeCheck(self, isoPath):
        if not isoPath.startswith("/"):
            logging.error("[DBUS]: isoPath[%s] is not abs path.", isoPath)
            return

        if self.methodLock:
            logging.error("[DBUS]: Please wait. Last method[%s] unfinished.", self.runningMethod)
            return
        self.methodLock = True
        self.runningMethod = "preUpgradeCheck"

        ret = self.VersionManagerHD.up_pre_check(isoPath)
        if ret != 0:
            self.daqInfo.send_all_info()

        self.methodLock = False
        self.runningMethod = None

    #执行升级
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature="s")
    def upgradeVersion(self, isoPath):
        if not isoPath.startswith("/"):
            logging.error("[DBUS]: isoPath[%s] is not abs path.", isoPath)
            return
        if self.methodLock:
            logging.error("[DBUS]: Please wait. Last method[%s] unfinished.", self.runningMethod)
            return
        self.methodLock = True
        self.runningMethod = "upgradeVersion"

        ret = self.VersionManagerHD.up_act(isoPath)
        if ret != 0:
            self.daqInfo.send_all_info()

        self.methodLock = False
        self.runningMethod = None


    # 获取当前版本
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='s')
    def getCurOsVersion(self):
        return self.osInfoHD.get_pretty_name()

    #还原前检测 un
    @dbus.service.method(RM_DBUS_INTERFACE)
    def preFallbackCheck(self):
        # 是否有还原文件系统
        if self.methodLock:
            logging.error("[DBUS]: Please wait. Last method[%s] unfinished.", self.runningMethod)
            return
        self.methodLock = True
        self.runningMethod = "preFallbackCheck"

        ret = self.VersionManagerHD.back_pre_check()
        if ret != 0:
            self.daqInfo.send_all_info()
        self.methodLock = False
        self.runningMethod = None


    #执行还原
    @dbus.service.method(RM_DBUS_INTERFACE)
    def fallbackVersion(self):
        if self.methodLock:
            logging.error("[DBUS]: Please wait. Last method[%s] unfinished.", self.runningMethod)
            return
        self.methodLock = True
        self.runningMethod = "fallbackVersion"

        ret = self.VersionManagerHD.back_act()
        if ret != 0:
            self.daqInfo.send_all_info()

        self.methodLock = False
        self.runningMethod = None

    # 解锁 for test
    @dbus.service.method(RM_DBUS_INTERFACE)
    def TEST_unlockMethod(self):
        self.methodLock = False
        self.runningMethod = None

    # 获取线程状态，
    # 阻塞 -- True
    # 在运行 -- False
    @dbus.service.method(RM_DBUS_INTERFACE)
    def isblocked(self):
        logging.info("[DBUS]: IS NOT BLOCKED")
        return False

    # 重启后，获取升级或还原的结果
    # 升级：成功、完成、失败
    # 还原：成功、失败
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='ss')
    def getState(self):
        return self.VersionManagerHD.get_state(), ""

    # 旧系统是否存在
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='i')
    def isOldOsExists(self):
        ret = self.VersionManagerHD.back_check_old_os_exists()
        return ret

    # test
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='s')
    def test_getOldOs(self):
        return self.VersionManagerHD.get_old_os()

    # 获取当前第三方应用列表
    # 返回值：应用列表
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='as')
    def getAppList(self):       
        logging.info("[DBUS]: get App List.")
        return self.VersionManagerHD.up_getAppList()

    # 设置迁移的第三方应用列表
    # 参数：列表
    # 返回值：0--成功， 其他--失败
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='as')
    def setMoveAppList(self, appList):
        logging.info("[DBUS]: set move App List [%s].", appList)
        self.VersionManagerHD.up_setAppList(appList)

    # 20240328 已无用
    # 清除不需要迁移的第三方应用列表
    @dbus.service.method(RM_DBUS_INTERFACE)
    def clearMoveAppList(self):
        logging.info("[DBUS]: clear move App List.")
        self.VersionManagerHD.up_clearMoveAppList()
    
    # 获取升级不兼容应用列表
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='as')
    def getUncompatPackageList(self):
        logging.info("[DBUS]: get uncompat App List.")
        return self.VersionManagerHD.get_uncompat_package()

    # 软件是否迁移的开关，默认为否
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='u')
    def switchAppMove(self, state):
        logging.info("[DBUS]: apps move switch [%s].", state)
        if state != True:
            state = False
        self.VersionManagerHD.switch_apps_move(state)

    # 获取是否超出还原时间
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='i')
    def isOverTime(self):
        logging.info("[DBUS]: get is overtime")
        ret = self.VersionManagerHD.is_overTime()
        logging.info("[DBUS]: get is overtime[%d]", ret)
        return ret

    # 前端显示了结果界面
    @dbus.service.method(RM_DBUS_INTERFACE)
    def addNoticeCount(self):
        self.VersionManagerHD.add_notice()

    # 安全设置
    # pwquality
    # userlock
    # exectl
    # devctl
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='as')
    def setSecurityList(self, kysecList):
        logging.info("[DBUS]: set not open security list.")
        self.VersionManagerHD.up_set_secList(kysecList)

    @dbus.service.method(RM_DBUS_INTERFACE)
    def clearSecurityList(self):
        logging.info("[DBUS]: clear security list.")
        self.VersionManagerHD.up_clear_secList()

    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='as')
    def getSecurityList(self):
        logging.info("[DBUS]: get security list.")
        return self.VersionManagerHD.up_get_secList()

    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='ss')
    def setActivationFiles(self, iPath, lPath):
        logging.info("[DBUS]: kyinfo is [%s] and LICENSE is [%s].", iPath, lPath)
        return self.VersionManagerHD.set_activation_file(iPath, lPath)

    # 废弃
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='as')
    def getFailedPackageList(self):
        return []

    # 新增 2024
    # 检测升级信息
    @dbus.service.method(RM_DBUS_INTERFACE)
    def checkUpgradeInfo(self):
        logging.info("[DBUS]: check upgrade info.")
        self.rmRemote.get_sources_info()

    # 检测升级信息进度信号
    @dbus.service.signal(RM_DBUS_INTERFACE, signature="iisas")
    def checkProcessSig(self, state, process, reason, content):
        logging.info("Emitting check state[%d] process[%d] reaseon[%s].", state, process, reason)

    # 获取可升级版本信息
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='as')
    def getAvailableOs(self):
        logging.info("[DBUS]: get available os.")
        return self.rmRemote.osOutList

    # 更新下载
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='ss')
    def startDownload(self, name, version):
        logging.info("[DBUS]: start download [%s] [%s].", name, version)
        self.rmRemote.start_download(name, version)

    # 重新下载
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='ss')
    def retryDownload(self, name, version):
        logging.info("[DBUS]: retry download [%s] [%s].", name, version)
        self.rmRemote.retry_download(name, version)

    # 取消下载
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='is')
    def cancelDownload(self):
        logging.info("[DBUS]: cancel download")
        return self.rmRemote.cancel_download()


    # 下载进度信号
    @dbus.service.signal(RM_DBUS_INTERFACE, signature="iisss")
    def downloadProcessSig(self, state, process, reason, speed, isoPath):
        logging.info("Emitting download state[%d] process[%d] reaseon[%s] speed[%s] isoPath[%s].", state, process, reason, speed, isoPath)


    # 设置服务端地址
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='s', out_signature='is')
    def setServerAddress(self, address):
        logging.info("[DBUS]: set address[%s].",address)
        self.rmRemote.set_remote_address(address)
        return 0, ''

    # 获取服务端地址
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='s')
    def getServerAddress(self):
        logging.info("[DBUS]: get address.")
        return self.rmRemote.get_remote_address()

    # 获取默认服务端地址
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='s')
    def getServerDefaultAddress(self):
        logging.info("[DBUS]: get default address.")
        return self.rmRemote.get_remote_default_address()

    # 删除备份
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='is')
    def deleteOldfs(self):
        logging.info("[DBUS]: delete oldfs.")
        ret, info = self.rmClear.manual_clear()
        return ret, info
    

    # 获取显示配置文件
    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='s')
    def getDisplayPath(self):
        logging.info("[DBUS]: get display path.")
        return self.rmRemote.get_base_display()
    
    # 检查ISO 是否已经下载完成
    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='ss')
    def checkIsoExist(self, name, version):
        logging.info("[DBUS]: check iso exist.")
        if len(name) == 0 or len(version) == 0:
            logging.warning("[DBUS] in param is [%s] [%s]", name, version)
            return False

        return self.rmRemote.check_iso_exist(name, version)

    @dbus.service.method(RM_DBUS_INTERFACE)
    def setAdvancePageDefault(self):
        logging.info("[DBUS]: set advance page default.")
        self.VersionManagerHD.up_set_default()

    @dbus.service.method(RM_DBUS_INTERFACE, in_signature='b')
    def setDownloadFormat(self, state):
        logging.info("[DBUS]: set download format.")
        self.rmRemote.set_download_format(state)

    @dbus.service.method(RM_DBUS_INTERFACE, out_signature='b')
    def getDownloadFormat(self):
        logging.info("[DBUS]: get download format.")
        return self.rmRemote.get_download_format()